package flux;

import flux.j2ee.J2eeAction;

import javax.jms.Message;
import java.io.Serializable;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;

/**
 * JMS Trigger.
 */
public interface JmsTrigger extends Trigger, J2eeAction {

  /**
   * @return the name of the queue we are listening to
   */
  String getListeningQueueName();

  /**
   * @return the polling delay between each attempt to fetch messages from the
   *         queue
   */
  String getPollingDelay();

  /**
   * Returns the JMS queue or topic factory.
   *
   * @return The JMS queue or topic factory.
   */
  String getConnectionFactory();

  /**
   * Returns the optional message selector.
   *
   * @return The optional message selector.
   */
  String getMessageSelector();

  /**
   * @return If <tt>true</tt>, we fetch all messages from the queue. If
   *         <tt>false</tt>, we just fetch one single message.
   */
  boolean isFetchAllMessages();

  /**
   * Sets the JMS queue or topic factory.
   *
   * @param connectionFactory The JMS queue or topic factory.
   */
  void setConnectionFactory(String connectionFactory);

  /**
   * @param fetchAll If set to <tt>true</tt>, we fetch all messages from the
   *                 queue. If set to <tt>false</tt>, we just fetch one single
   *                 message.
   */
  void setFetchAllMessages(boolean fetchAll);

  /**
   * @param queueName The name of the queue we are listening to
   */
  void setListeningQueueName(String queueName);

  /**
   * Sets the optional message selector.
   *
   * @param messageSelector The optional message selector.
   */
  void setMessageSelector(String messageSelector);

  /**
   * @param timeExpression the polling delay between each attempt to fetch messages
   *                       from the queue
   */
  void setPollingDelay(String timeExpression);


  /**
   * JmsTrigger Result
   */
  public class JmsTriggerResult implements Serializable, Cloneable {

    public List<Message> messages;
    public Date fetched;

    public JmsTriggerResult() {
      this(null);
    }

    public JmsTriggerResult(List<Message> messages) {
      this.messages = messages;
      fetched = new Date();
    }

    public void addMessage(Message message) {
      if (this.messages == null)
        this.messages = new LinkedList<Message>();
      messages.add(message);
    }

    public List<Message> getMessages() {
      return this.messages;
    }

    public Date getMessagesReceived() {
      return this.fetched;
    }

    public int size() {
      if (this.messages != null) {
        return this.messages.size();
      } else
        return 0;
    }
  }
}