package flux;

/**
 * RabbitMQAction that publishes messages to RabbitMQ queues.
 * This works with AMQP server that implements 0-9-1 version
 * of the specification.
 *
 * @author arul@flux.ly
 */
public interface RabbitMQAction extends RabbitMQ, Action {

  /**
   * RabbitMQ exchange name.
   *
   * @param exchangeName exchange name.
   */
  public void setExchangeName(String exchangeName);

  /**
   * RabbitMQ exchange type: DIRECT, FANOUT, TOPIC, HEADERS.
   * Default is DIRECT.
   *
   * @param exchangeType exchange type.
   */
  public void setExchangeType(String exchangeType);

  /**
   * Sets the queue type: EXCLUSIVE, SHARED.
   * Default is SHARED. Queues are durable.
   *
   * @param queueType queue type.
   */
  public void setQueueType(String queueType);

  /**
   * Sets the routing key.
   *
   * @param routingKey routing key.
   */
  public void setRoutingKey(String routingKey);

  /**
   * Sets the message to be published to the queue.
   *
   * @param message message.
   */
  public void setMessage(String message);
}
