package flux;

import java.io.Serializable;
import java.util.Properties;

/**
 * TwitterAction that invokes OAuth protected resources.
 *
 * @author arul@flux.ly
 */
public interface TwitterAction extends Action {

  /**
   * Sets the HTTP action used to access resources. Default is GET.
   *
   * @param actionType HTTP Verb used to access resources.
   */
  public void setRequestMethod(RestActionType actionType);

  /**
   * Sets Twitter Application consumer key.
   *
   * @param apiKey OAuth consumer key.
   */
  public void setConsumerKey(String apiKey);

  /**
   * Sets the number of records to retrieve.
   *
   * @param count the number of records to retrieve.
   */
  public void setCount(int count);

  /**
   * Sets Twitter Application consumer secret.
   *
   * @param apiSecret OAuth consumer secret.
   */
  public void setConsumerSecret(String apiSecret);

  /**
   * Sets OAuth for protected resources. Defaults to false.
   *
   * @param oAuthEnabled OAuth for protected resources.
   */
  public void setOAuthEnabled(boolean oAuthEnabled);

  /**
   * Sets the page of results to retrieve.
   *
   * @param page the page of results to retrieve.
   */
  public void setPage(int page);

  /**
   * Sets the body parameters.
   *
   * @param parameters Body parameters.
   */
  public void setParameters(Properties parameters);

  /**
   * Sets the password of the consumer.
   *
   * @param password Password of the consumer.
   */
  public void setPassword(String password);

  /**
   * Sets the resource URL.
   * For example, http://api.twitter.com/1/statuses/mentions.json or
   * http://api.twitter.com/1/direct_messages/new.xml
   *
   * @param url resource URL.
   */
  public void setResourceUrl(String url);

  /**
   * Sets the search query.
   *
   * @param searchQuery Search query.
   */
  public void setSearchQuery(String searchQuery);

  /**
   * Sets the username of the consumer.
   *
   * @param username Username of the consumer.
   */
  public void setUsername(String username);

  /**
   * The response from the TwitterAction.
   */
  public class TwitterActionResult implements Serializable, Cloneable {
    /**
     * HTTP response entity.
     */
    public String response;
    /**
     * HTTP response code.
     */
    public int responseCode;

    public Object clone() throws CloneNotSupportedException {
      return super.clone();
    } // clone()

    public String toString() {
      return super.toString();
    } // toString()
  }
}
