package flux;

import java.util.List;

/**
 * Untars a specified tar archive file. It supports
 * decompressing bzip, gzip and tar file types.
 *
 * @author arul@flux.ly
 */
public interface UntarFileAction extends Action {

  /**
   * Sets the directory where to store the expanded files.
   *
   * @param dest The directory where to store the expanded files.
   */
  public void setDestination(String dest);

  /**
   * Sets the file exclude patterns. It follows ant style regex
   * file exclude patterns.
   *
   * @param excludePatterns The file exclude patterns.
   */
  public void setExcludePatterns(List<String> excludePatterns);

  /**
   * Sets the file include patterns. It follows ant style regex
   * file include patterns.
   *
   * @param includePatterns The file include patterns.
   */
  public void setIncludePatterns(List<String> includePatterns);

  /**
   * Overwrite files, even if they are newer than the corresponding
   * entries in the archive. Default is true.
   *
   * @param overwrite Overwrite files, even if they are newer than
   * the corresponding entries in the archive.
   */
  public void setOverwriteEnabled(boolean overwrite);
  /**
   * Sets the archive file to expand.
   *
   * @param src The archive file to expand.
   */
  public void setSource(String src);
}
