package fluximpl;

import flux.*;
import org.apache.cxf.endpoint.Server;
import org.apache.cxf.transport.http_jetty.JettyHTTPServerEngine;

import javax.jws.WebParam;
import javax.jws.WebService;
import javax.xml.bind.annotation.adapters.XmlJavaTypeAdapter;
import java.rmi.RemoteException;
import java.util.Map;

/**
 * FlowChartListener WebService implementation.
 *
 * @author arul@flux.ly
 */
@WebService(endpointInterface = "fluximpl.FlowChartListener", serviceName = "FlowChartListener")
public class FlowChartListenerImpl implements FlowChartListener {

  public static final String LISTENER_EXPEDITED = "LISTENER_EXPEDITED";
  public static final String LISTENER_VARIABLE = "LISTENER_VARIABLE";
  private FlowContext flowContext;
  private Server soapServer;
  private Server restServer;
  private JettyHTTPServerEngine engine;
  private boolean running = true;
  private boolean secured = true;
  private String username;
  private String password;

  public FlowChartListenerImpl(FlowContext flowContext, String username, String password) {
    this.flowContext = flowContext;
    this.username = username;
    this.password = password;
    try {
      this.secured = flowContext.getEngine().isSecured();
    } catch (RemoteException e) {
      // ignore
    }
  }

  public void setSoapServer(Server soapServer) {
    this.soapServer = soapServer;
  }

  public void setRestServer(Server restServer) {
    this.restServer = restServer;
  }

  public void setEngine(JettyHTTPServerEngine engine) {
    this.engine = engine;
  }

  @Override
  public long expedite() {
    long count = 0;
    FlowChart flowChart = flowContext.getFlowChart();
    flowChart.getVariableManager().put(LISTENER_EXPEDITED, Boolean.TRUE);
    try {
      if (secured) {
        LocalSecurity rs = Factory.makeInstance().makeLocalSecurity();
        rs.login(flowContext.getEngine(), username, password);
        flowContext.getEngine().put(flowChart);
        count = flowContext.getEngine().expedite(flowChart.getName());
        rs.logout(flowContext.getEngine());
      } else {
        flowContext.getEngine().put(flowChart);
        count = flowContext.getEngine().expedite(flowChart.getName());
      }
    } catch (Exception e) {
      // ignore
    }
    cleanup();
    running = false;
    return count;
  }

  @Override
  public long expedite(@WebParam(name = "variables") @XmlJavaTypeAdapter(StringMapAdapter.class) Map<String, String> variables) {
    long count = 0;
    FlowChart flowChart = flowContext.getFlowChart();
    flowChart.getVariableManager().put(LISTENER_EXPEDITED, Boolean.TRUE);
    flowChart.getVariableManager().put(LISTENER_VARIABLE, variables);
    try {
      if (secured) {
        LocalSecurity rs = Factory.makeInstance().makeLocalSecurity();
        rs.login(flowContext.getEngine(), username, password);
        flowContext.getEngine().put(flowChart);
        count = flowContext.getEngine().expedite(flowChart.getName());
        rs.logout(flowContext.getEngine());
      } else {
        flowContext.getEngine().put(flowChart);
        count = flowContext.getEngine().expedite(flowChart.getName());
      }
    } catch (Exception e) {
      // ignore
    }
    cleanup();
    running = false;
    return count;
  }

  @Override
  public void dispose() {
    cleanup();
    running = false;
  }

  @Override
  public boolean isRunning() {
    return running;
  }

  private void cleanup() {
    ShutdownHandler handler = new ShutdownHandler();
    handler.start();
  }

  class ShutdownHandler extends Thread {
    @Override
    public void run() {
      try {
        Thread.sleep(1000);
      } catch (InterruptedException e) {
        // ignore
      }
      if (soapServer != null) {
        soapServer.destroy();
      }
      if (restServer != null) {
        restServer.destroy();
      }
      try {
        engine.shutdown();
      } catch (Exception e) {
        // ignore
      }
    }
  }
}
