package fluximpl;

import java.awt.*;
import java.beans.BeanDescriptor;
import java.beans.BeanInfo;
import java.beans.PropertyDescriptor;
import java.util.Arrays;
import java.util.Vector;

/**
 * BeanInfo for MercurialTrigger.
 *
 * @author arul@flux.ly
 */
public class MercurialTriggerImplBeanInfo extends ActionImplBeanInfo {
  /**
   * A small image for this custom action to be displayed in the Flux GUI.
   */
  private Image smallImage;

  /**
   * A large image for this custom action to be displayed in the Flux GUI.
   */
  private Image bigImage;

  protected BeanDescriptor bd = new BeanDescriptor(MercurialTriggerImpl.class);

  public MercurialTriggerImplBeanInfo(BeanDescriptor bd) {
    super();
    this.bd = bd;
    setCategory(bd, ActionImplBeanInfo.CORE);
  }

  public MercurialTriggerImplBeanInfo() {
    super();
    bd.setDisplayName("Mercurial Trigger");
    bd.setShortDescription("Fires when a new changeset is available.");
    setCategory(bd, ActionImplBeanInfo.CORE);
  }

  public Image getIcon(int type) {
    if (type == BeanInfo.ICON_COLOR_16x16) {
      return getSmallImage();
    } // if
    if (type == BeanInfo.ICON_COLOR_32x32) {
      return getBigImage();
    } // if
    return null;
  } // getIcon()

  /**
   * Returns a small image that represents this JavaBean.
   *
   * @return A small image that represents this JavaBean.
   */
  private Image getSmallImage() {
    if (smallImage == null) {
      smallImage = loadImage("/MercurialTriggerImpl_small.png");
    } // if
    return smallImage;
  } // getSmallImage()

  /**
   * Returns a large image that represents this JavaBean.
   *
   * @return A large image that represents this JavaBean.
   */
  private Image getBigImage() {
    if (bigImage == null) {
      bigImage = loadImage("/MercurialTriggerImpl_big.png");
    } // if
    return bigImage;
  } // getBigImage()


  public BeanDescriptor getBeanDescriptor() {
    return bd;
  }

  public PropertyDescriptor[] getPropertyDescriptors() {
    Vector<PropertyDescriptor> descriptors = new Vector<PropertyDescriptor>(Arrays.asList(super.getPropertyDescriptors()));
    try {
      int order = 10;
      descriptors.add(makePropertyDescriptor("repositoryUrl", "Repository URL", MercurialTriggerImpl.class.getName(), "getRepositoryUrl", true, null, order++, false, false, false, false));
      descriptors.add(makePropertyDescriptor("path", "Path", MercurialTriggerImpl.class.getName(), "getPath", false, null, order++, false, false, false, false));
      descriptors.add(makePropertyDescriptor("pollingDelay", "Polling Delay", MercurialTriggerImpl.class.getName(), "getPollingDelay", false, null, order++, false, false, false, false));
    } catch (Exception e) {
      e.printStackTrace();
      throw new IllegalStateException("Could not create property descriptor.");
    } // catch

    return descriptors.toArray(new PropertyDescriptor[descriptors.size()]);
  }

  public void setCategory(BeanDescriptor beanDescriptor, String category) {
    beanDescriptor.setValue("category", category);
  }
}