package fluximpl;

import java.io.Serializable;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.TextMessage;

/**
 * Minimalistic POJO that contains body of the JMS text message, correlation id, message id.
 * Just add what you need for persistence. No bloating flow context.
 *
 * @author Copyright 2012 Flux Corporation. All rights reserved.
 */
public class TextMessageImpl implements TextMessage, Serializable, Cloneable {

  public String messageId;
  public int priority;
  public String correlationId;
  public String text;
  public long timestamp;
  public Map<String, String> properties = null;

  public TextMessageImpl() {
    try {
      clearBody();
      setJMSCorrelationID("");
      setJMSMessageID("");
    } catch (JMSException e) {
      e.printStackTrace();
    }
    properties = new HashMap<String, String>();

  }

  public String getText() throws JMSException {
    return text;
  }

  public void setText(String text) throws JMSException {
    this.text = text;
  }

  public void acknowledge() throws JMSException {
    throw new UnsupportedOperationException("Not supported!");
  }

  public void clearBody() throws JMSException {
    this.text = "";
  }

  public void clearProperties() throws JMSException {
    if (properties != null) properties.clear();
  }

  public boolean getBooleanProperty(String arg0) throws JMSException {
    throw new UnsupportedOperationException("Not supported!");
  }

  public byte getByteProperty(String arg0) throws JMSException {
    throw new UnsupportedOperationException("Not supported!");
  }

  public double getDoubleProperty(String arg0) throws JMSException {
    throw new UnsupportedOperationException("Not supported!");
  }

  public float getFloatProperty(String arg0) throws JMSException {
    throw new UnsupportedOperationException("Not supported!");
  }

  public int getIntProperty(String arg0) throws JMSException {
    throw new UnsupportedOperationException("Not supported!");
  }

  public String getJMSCorrelationID() throws JMSException {
    return this.correlationId;
  }

  public byte[] getJMSCorrelationIDAsBytes() throws JMSException {
    return this.correlationId.getBytes();
  }

  public int getJMSDeliveryMode() throws JMSException {
    return Message.DEFAULT_DELIVERY_MODE;
  }

  public Destination getJMSDestination() throws JMSException {
    return null;
  }

  public long getJMSExpiration() throws JMSException {
    return Message.DEFAULT_TIME_TO_LIVE;
  }

  public String getJMSMessageID() throws JMSException {
    return this.messageId;
  }

  public int getJMSPriority() throws JMSException {
    return priority;
  }

  public boolean getJMSRedelivered() throws JMSException {
    return false;
  }

  public Destination getJMSReplyTo() throws JMSException {
    return null;
  }

  public long getJMSTimestamp() throws JMSException {
    return timestamp;
  }

  public String getJMSType() throws JMSException {
    return TextMessage.class.getSimpleName();
  }

  public long getLongProperty(String name) throws JMSException {
    throw new UnsupportedOperationException("Not supported!");
  }

  public Object getObjectProperty(String name) throws JMSException {
    if (this.properties != null) return properties.get(name);
    return null;
  }

  public Enumeration<String> getPropertyNames() throws JMSException {
    if (this.properties != null) {
      return new Enumeration<String>() {
        private Iterator<String> iterator = properties.keySet().iterator();

        public boolean hasMoreElements() {
          return iterator.hasNext();
        }

        public String nextElement() {
          return iterator.next();
        }

      };
    }
    return null;
  }

  public short getShortProperty(String name) throws JMSException {
    throw new UnsupportedOperationException("Not supported!");
  }

  public String getStringProperty(String name) throws JMSException {
    return this.properties.get(name);
  }

  public boolean propertyExists(String name) throws JMSException {
    return this.properties.containsKey(name);
  }

  public void setBooleanProperty(String name, boolean value) throws JMSException {
    throw new UnsupportedOperationException("Not supported!");
  }

  public void setByteProperty(String name, byte value) throws JMSException {
    throw new UnsupportedOperationException("Not supported!");
  }

  public void setDoubleProperty(String name, double value) throws JMSException {
    throw new UnsupportedOperationException("Not supported!");
  }

  public void setFloatProperty(String name, float value) throws JMSException {
    throw new UnsupportedOperationException("Not supported!");
  }

  public void setIntProperty(String name, int value) throws JMSException {
    throw new UnsupportedOperationException("Not supported!");
  }

  public void setJMSCorrelationID(String correlationID) throws JMSException {
    this.correlationId = correlationID;
  }

  public void setJMSCorrelationIDAsBytes(byte[] correlationID) throws JMSException {
    this.correlationId = new String(correlationID);
  }

  public void setJMSDeliveryMode(int deliveryMode) throws JMSException {
    throw new UnsupportedOperationException("Not supported!");
  }

  public void setJMSDestination(Destination destination) throws JMSException {
    throw new UnsupportedOperationException("Not supported!");
  }

  public void setJMSExpiration(long expiration) throws JMSException {
    throw new UnsupportedOperationException("Not supported!");
  }

  public void setJMSMessageID(String id) throws JMSException {
    this.messageId = id;
  }

  public void setJMSPriority(int priority) throws JMSException {
    this.priority = priority;
  }

  public void setJMSRedelivered(boolean redelivered) throws JMSException {
    throw new UnsupportedOperationException("Not supported!");
  }

  public void setJMSReplyTo(Destination replyTo) throws JMSException {
    throw new UnsupportedOperationException("Not supported!");
  }

  public void setJMSTimestamp(long timestamp) throws JMSException {
    this.timestamp = timestamp;
  }

  public void setJMSType(String type) throws JMSException {
    throw new UnsupportedOperationException("Not supported!");
  }

  public void setLongProperty(String name, long value) throws JMSException {
    throw new UnsupportedOperationException("Not supported!");
  }

  public void setObjectProperty(String name, Object value) throws JMSException {
    if (value instanceof String) {
      this.properties.put(name, (String) value);
    } else {
      throw new UnsupportedOperationException("Not supported!");
    }
  }

  public void setShortProperty(String name, short value) throws JMSException {
    throw new UnsupportedOperationException("Not supported!");
  }

  public void setStringProperty(String name, String value) throws JMSException {
    this.properties.put(name, value);
  }

  public static TextMessage makeActiveMQTextMessage(TextMessage message) throws JMSException {
    return makeMessage(message, message.getText());
  }

  public static Message makeActiveMQTextMessage(Message message) throws JMSException {
    String body = "";
    //check if this message is a text message
    if (message instanceof TextMessage) {
      body = ((TextMessage) message).getText();
    }
    return makeMessage(message, body);
  }

  @SuppressWarnings("unchecked")
  private static TextMessage makeMessage(Message message, String body) throws JMSException {
    final TextMessageImpl msg = new TextMessageImpl();

    //general fields
    msg.setJMSMessageID(message.getJMSMessageID());
    msg.setJMSCorrelationID(message.getJMSCorrelationID());
    msg.setText(body);
    msg.setJMSPriority(message.getJMSPriority());
    msg.setJMSTimestamp(message.getJMSTimestamp());

    //properties
    Enumeration keys = message.getPropertyNames();
    while (keys.hasMoreElements()) {
      String key = (String) keys.nextElement();
      String val = message.getStringProperty(key);
      if (val != null) {
        msg.setStringProperty(key, val);
      } else {
        System.out.println("Skipping non-String Property '" + key + "'.");
      }
    }

    return msg;
  }

  @Override
  public Object clone() throws CloneNotSupportedException {
    try {
      return makeActiveMQTextMessage(this);
    } catch (JMSException e) {
      throw new CloneNotSupportedException("Clone failed, root cause is " + e.getMessage());
    }
  }

  @Override
  public boolean equals(Object o) {
    if (this == o) return true;
    if (o == null || getClass() != o.getClass()) return false;

    TextMessageImpl that = (TextMessageImpl) o;

    if (priority != that.priority) return false;
    if (timestamp != that.timestamp) return false;
    if (correlationId != null ? !correlationId.equals(that.correlationId) : that.correlationId != null) return false;
    if (messageId != null ? !messageId.equals(that.messageId) : that.messageId != null) return false;
    if (properties != null ? !properties.equals(that.properties) : that.properties != null) return false;
    if (text != null ? !text.equals(that.text) : that.text != null) return false;

    return true;
  }

  @Override
  public int hashCode() {
    int result = messageId != null ? messageId.hashCode() : 0;
    result = 31 * result + priority;
    result = 31 * result + (correlationId != null ? correlationId.hashCode() : 0);
    result = 31 * result + (text != null ? text.hashCode() : 0);
    result = 31 * result + (int) (timestamp ^ (timestamp >>> 32));
    result = 31 * result + (properties != null ? properties.hashCode() : 0);
    return result;
  }
}