package fluximpl;

import java.awt.*;
import java.beans.BeanDescriptor;
import java.beans.BeanInfo;
import java.beans.PropertyDescriptor;
import java.util.Arrays;
import java.util.Vector;

/**
 * BeanInfo for WebServiceTrigger.
 *
 * @author arul@flux.ly
 */
public class WebServiceTriggerImplBeanInfo extends ActionImplBeanInfo {
  /**
   * A small image for this custom action to be displayed in the Flux GUI.
   */
  private Image smallImage;

  /**
   * A large image for this custom action to be displayed in the Flux GUI.
   */
  private Image bigImage;

  protected BeanDescriptor bd = new BeanDescriptor(WebServiceTriggerImpl.class);

  public WebServiceTriggerImplBeanInfo(BeanDescriptor bd) {
    super();
    this.bd = bd;
    setCategory(bd, ActionImplBeanInfo.CORE);
  }

  public WebServiceTriggerImplBeanInfo() {
    super();
    bd.setDisplayName("Web Service Trigger");
    bd.setShortDescription("Fires when a SOAP or HTTP request is received.");
    setCategory(bd, ActionImplBeanInfo.CORE);
  }

  public Image getIcon(int type) {
    if (type == BeanInfo.ICON_COLOR_16x16) {
      return getSmallImage();
    } // if
    if (type == BeanInfo.ICON_COLOR_32x32) {
      return getBigImage();
    } // if
    return null;
  } // getIcon()

  /**
   * Returns a small image that represents this JavaBean.
   *
   * @return A small image that represents this JavaBean.
   */
  private Image getSmallImage() {
    if (smallImage == null) {
      smallImage = loadImage("/WebServiceTriggerImpl_small.png");
    } // if
    return smallImage;
  } // getSmallImage()

  /**
   * Returns a large image that represents this JavaBean.
   *
   * @return A large image that represents this JavaBean.
   */
  private Image getBigImage() {
    if (bigImage == null) {
      bigImage = loadImage("/WebServiceTriggerImpl_big.png");
    } // if
    return bigImage;
  } // getBigImage()


  public BeanDescriptor getBeanDescriptor() {
    return bd;
  }

  public PropertyDescriptor[] getPropertyDescriptors() {
    Vector<PropertyDescriptor> descriptors = new Vector<PropertyDescriptor>(Arrays.asList(super.getPropertyDescriptors()));
    try {
      int order = 10;
      descriptors.add(makePropertyDescriptor("port", "Port", WebServiceTriggerImpl.class.getName(), "getPort", true, null, order++, false, false, false, false));
      descriptors.add(makePropertyDescriptor("serviceName", "Service Name", WebServiceTriggerImpl.class.getName(), "getServiceName", true, null, order++, false, false, false, false));
      descriptors.add(makePropertyDescriptor("username", "Username", WebServiceTriggerImpl.class.getName(), "getUsername", false, null, order++, false, false, false, false));
      descriptors.add(makePropertyDescriptor("password", "Password", WebServiceTriggerImpl.class.getName(), "getPassword", false, null, order++, false, false, false, true));
      descriptors.add(makePropertyDescriptor("engineUsername", "Engine Username", WebServiceTriggerImpl.class.getName(), "getEngineUsername", false, null, order++, false, false, false, false));
      descriptors.add(makePropertyDescriptor("enginePassword", "Engine Password", WebServiceTriggerImpl.class.getName(), "getEnginePassword", false, null, order++, false, false, false, true));
    } catch (Exception e) {
      e.printStackTrace();
      throw new IllegalStateException("Could not create property descriptor.");
    } // catch

    return descriptors.toArray(new PropertyDescriptor[descriptors.size()]);
  }

  public void setCategory(BeanDescriptor beanDescriptor, String category) {
    beanDescriptor.setValue("category", category);
  }
}